# Warewulf shared library
#
# This script is a component of Warewulf,
# http://www.runlevelzero.net/greg/warewulf
#
#########################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The GNU GPL Document can be found at:
# http://www.gnu.org/copyleft/gpl.html
#
#########################################################################
#
# Written and maintained by:
#       Greg Kurtzer, <gmkurtzer@lbl.gov>

my $debug = ();
#$debug = 1;

package Warewulf::PXE;
use Warewulf::Config;
use Warewulf::Util;
use Exporter;
use File::Path;
@ISA = ('Exporter');

@EXPORT = qw (
   &pxe_update
);

my $pxe_template = "default ww
label ww
kernel /#kernel image#
append rw initrd=#wwinitrd image# root=/dev/root ramdisk=20480 #warewulf#
";


sub pxe_update {
   my ( $verbose, @null ) = @_;
   my %nodes = &node_config();
   my %master = &master_config();
   my $pxecfgdir;
   my $pxelinux_found;
   if ( $master{"paths"}{"tftpboot"} ) {
      $pxecfgdir = "$master{'paths'}{'tftpboot'}/pxelinux.cfg";
   } else {
      $pxecfgdir = "/tftpboot/pxelinux.cfg";
   }
   my $gw = "";

   if ( -d $pxecfgdir ) {
      system("rm -f $pxecfgdir/*");
   } else {
      system("mkdir -p $pxecfgdir");
   }

   if ( ! -f "/tftpboot/pxelinux.0" ) {
      if ( $verbose ) {
         print "   Looking for pxelinux.0...\n";
      }
      open(PXELINUX,"locate pxelinux.0 2>/dev/null |");
      while(<PXELINUX>) {
         chomp;
         if ( $_ =~ /syslinux\/pxelinux\.0$/ ) {
            if ( $verbose ) {
               print "   Found $_\n";
            }
            $pxelinux_found = "1";
            system("cp -ap $_ /tftpboot/");
         }
      }
      if ( ! $pxelinux_found ) {
         if ( -f "/usr/lib/syslinux/pxelinux.0" ) {
            if ( $verbose ) {
               print "   Found /usr/lib/syslinux/pxelinux.0\n";
            }
            $pxelinux_found = "1";
            system("cp -ap /usr/lib/syslinux/pxelinux.0 /tftpboot/");
         }
      }
      if ( ! $pxelinux_found ) {
         if ( -f "/usr/lib64/syslinux/pxelinux.0" ) {
            if ( $verbose ) {
               print "   Found /usr/lib64/syslinux/pxelinux.0\n";
            }
            $pxelinux_found = "1";
            system("cp -ap /usr/lib64/syslinux/pxelinux.0 /tftpboot/");
         }
      }
      if ( ! $pxelinux_found ) {
         warn "WARNING: Could not find pxelinux.0! Please locate this, and copy\n";
         warn "         it to /tftpboot/!\n";
      }
   }

   if ( $verbose ) {
      print "   Adding default entry for new nodes\n";
   }

   open(PXEDEFAULT, "> $pxecfgdir/default");
   my $tmp = $pxe_template;
   my $boot_ip = $master{'ipaddrs'}{'boot'};
   $tmp =~ s/#kernel image#/kernel/g;
   $tmp =~ s/#wwinitrd image#/wwinitrd.img/g;
   $tmp =~ s/#warewulf#/warewulf=undef,$boot_ip/g;
   print PXEDEFAULT $tmp;
   close PXEDEFAULT;

   nodeloop: foreach $node ( sort keys %nodes ) {
      next nodeloop unless ( $nodes{$node}{"hardware addr"} =~ /\w\w:\w\w:\w\w:\w\w:\w\w/ );
      next nodeloop if ( $nodes{$node}{"group"} eq "new" );
      next nodeloop if ( $nodes{$node}{"group"} eq "ignore" );
      next nodeloop if ( $nodes{$node}{"group"} eq "disabled" );
      $wwnet = "warewulf=$node,";
      $wwnet .= "$master{'network'}{'admin ipaddr'},";
      if ( $nodes{$node}{'gateway ip'} ) {
         $wwnet .= "$nodes{$node}{'gateway ip'},";
      } else {
         $wwnet .= "$master{'network'}{'admin ipaddr'},";
      }
      $wwnet .= "$nodes{$node}{'vnfs'},";
      $wwnet .= "$nodes{$node}{'admin device'}=$nodes{$node}{'admin ipaddr'}:$master{'network'}{'admin netmask'}";
      if ( $nodes{$node}{'cluster device'} ne $nodes{$node}{'admin device'} ) {
         $wwnet .= ",$nodes{$node}{'cluster device'}=$nodes{$node}{'cluster ipaddr'}:$master{'network'}{'cluster netmask'}";
      }
      if ( $nodes{$node}{'sharedfs device'} ne $nodes{$node}{'admin device'} and
           $nodes{$node}{'sharedfs device'} ne $nodes{$node}{'cluster device'} ) {
         $wwnet .= ",$nodes{$node}{'sharedfs device'}=$nodes{$node}{'sharedfs ipaddr'}:$master{'network'}{'sharedfs netmask'}";
      }
      if ( $nodes{$node}{'other dev'} and $nodes{$node}{'other addr'} ) {
         $wwnet .= ",$nodes{$node}{'other dev'}=$nodes{$node}{'other addr'}";
      }

   # Convert the MAC address to lower case, replace colons with dashes and
   # prepend with 01- to indicate Ethernet... all this for PXELINUX >= 2.05
      $pxeid = lc("01-$nodes{$node}{'hardware addr'}");
      $pxeid =~ s/:/-/g;
      if ( $verbose ) {
         print "   [$node] $pxeid ($nodes{$node}{'group desc'})\n";
      }
      $tmp = $pxe_template;
      $tmp =~ s/#kernel image#/$nodes{$node}{'kernel image'}/g;
      $tmp =~ s/#wwinitrd image#/$nodes{$node}{'wwinitrd image'}/g;
      $tmp =~ s/#warewulf#/$wwnet/g;
      open(PXECFG, "> $pxecfgdir/$pxeid");
      print PXECFG $tmp;
      close PXECFG;

      if ( $nodes{$node}{'hardware addr2'} ) {
         if ( $verbose ) {
            print "   [$node] $nodes{$node}{'hardware addr2'} -> $pxeid\n";
         }
         $pxeid2 = lc("01-$nodes{$node}{'hardware addr2'}");
         $pxeid2 =~ s/:/-/g;
         #open(PXECFG, "> $pxecfgdir/$pxeid");
         #print PXECFG $tmp;
         #close PXECFG;
         link("$pxecfgdir/$pxeid", "$pxecfgdir/$pxeid2");
      }
   }
}


1;
