# Warewulf shared library
#
# This script is a component of Warewulf,
# http://www.runlevelzero.net/greg/warewulf
#
#########################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The GNU GPL Document can be found at:
# http://www.gnu.org/copyleft/gpl.html
#
#########################################################################
#
# Written and maintained by:
#       Greg Kurtzer, <gmkurtzer@lbl.gov>

my $debug = ();
#$debug = 1;

package Warewulf::Util;
use Warewulf::Config;
use Socket;
use File::Basename;
use Exporter;
@ISA = ('Exporter');

@EXPORT = qw (
   &gen_hosts
   &write_hosts
   &generate_random_string
   &users_filter
   &node_addr
   &node_addr_start
);

sub gen_hosts {
   my ( $ref_nodes, $ref_master, @null ) = @_;
   my %nodes = %$ref_nodes;
   my %master = %$ref_master;
   my %used;
   my $hsts;

   sub gen_hosts_line {
      my ( $node, $whichnet, $first, $last, $ref_ips, @null ) = @_;
      my %ips = %$ref_ips;
      my $rtn = "$ips{$whichnet}\t$first";
      if ( $ips{'admin'} eq $ips{$whichnet} ) {
         $rtn .= "${node}-admin ";
         $used{'admin'} = 1;
      }
      if ( $ips{'cluster'} eq $ips{$whichnet} ) {
         $rtn .= "${node}-clust ";
         $used{'cluster'} = 1;
      }
      if ( $ips{'sharedfs'} eq $ips{$whichnet} ) {
         $rtn .= "${node}-sharedfs";
         $used{'sharedfs'} = 1;
      }
      $rtn .= "$last\n";
      return $rtn;
   }

   $hsts .= "# Do not edit below this line, it will be overwritten by Warewulf! #\n";

   $hsts .= "# Master node configuration\n";

   my $n = $master{'network'}{'nodename'};
   if ( $master{'network'}{'default network'} eq "admin" ) {
      $hsts .= "$master{'network'}{'admin ipaddr'}\t$n $n-admin\n";
   } else {
      $hsts .= "$master{'network'}{'admin ipaddr'}\t$n-admin\n";
   }
   if ( $master{'network'}{'default network'} eq "cluster" ) {
      $hsts .= "$master{'network'}{'cluster ipaddr'}\t$n $n-cluster\n";
   } else {
      $hsts .= "$master{'network'}{'cluster ipaddr'}\t$n-cluster\n";
   }
   if ( $master{'network'}{'default network'} eq "sharedfs" ) {
      $hsts .= "$master{'network'}{'sharedfs ipaddr'}\t$n $n-sharedfs\n";
   } else {
      $hsts .= "$master{'network'}{'sharedfs ipaddr'}\t$n-sharedfs\n";
   }

   foreach $node ( sort keys %nodes ) {
      next if ( $nodes{$node}{"group"} eq "new" );
      next if ( $nodes{$node}{"group"} eq "ignore" );
      next if ( $nodes{$node}{"group"} eq "disabled" );
      $hsts .= "\n# Node '$node' configuration (group: $nodes{$node}{'group'})\n";
      if ( $master{'network'}{'default network'} eq "admin" ) {
         $hsts .= "$nodes{$node}{'admin ipaddr'}\t$node $node-admin\n";
      } else {
         $hsts .= "$nodes{$node}{'admin ipaddr'}\t$node-admin\n";
      }
      if ( $master{'network'}{'default network'} eq "cluster" ) {
         $hsts .= "$nodes{$node}{'cluster ipaddr'}\t$node $node-cluster\n";
      } else {
         $hsts .= "$nodes{$node}{'cluster ipaddr'}\t$node-cluster\n";
      }
      if ( $master{'network'}{'default network'} eq "sharedfs" ) {
         $hsts .= "$nodes{$node}{'sharedfs ipaddr'}\t$node $node-sharedfs\n";
      } else {
         $hsts .= "$nodes{$node}{'sharedfs ipaddr'}\t$node-sharedfs\n";
      }
   }

   return $hsts;
}

sub write_hosts {

   my %nodes = &Warewulf::Config::node_config();
   my %master = &Warewulf::Config::master_config();
   my $outfile;

   open(OUTFILE, "< /etc/hosts")
      or die "Could not open /etc/hosts for reading!\n";
   while (<OUTFILE>) {
      chomp;
      if ( $_ eq '# Warewulf nodes #' ) {
         last;
      } else {
         $outfile .= "$_\n";
      }
   }
   close OUTFILE;

   $outfile .= "# Warewulf nodes #\n";
   $outfile .= &gen_hosts(\%nodes, \%master);

   open(OUTFILE, "> /etc/hosts")
      or die "Could not open /etc/hosts for writing!\n";
   print OUTFILE "$outfile\n";
   close OUTFILE;

   my $masterIP = $master{"network"}{"$master{'network'}{'default network'} ipaddr"};
   my $null;
   my @master_addrs;
   my $iaddr;
   my $straddr;
   my $masterIP_OK;
   ( $null, $null, $null, $null, @master_addrs) = gethostbyname($master{'network'}{'nodename'});
   foreach $iaddr ( @master_addrs ) {
      $straddr = inet_ntoa($iaddr);
      if ( $straddr eq $masterIP ) {
         $masterIP_OK = 1;
      }
   }
   if ( ! $masterIP_OK ) {
      warn "\n\nWarning:\n";
      warn "The master's nodename '$master{'network'}{'nodename'}' resolved to $straddr.\n";
      warn "It should resolve to $masterIP instead.  This should be fixed.\n";
      warn "Remove the extranious entry above '# Warewlf nodes #' in /etc/hosts.\n\n";
   }
}



sub generate_random_string {
    my $size = shift;
    my @alphanumeric = ('a'..'z', 'A'..'Z', 0..9);
    my $randstring = join '', map $alphanumeric[rand @alphanumeric], 0..$size;
    return $randstring;
}

sub users_filter {
# PARAMS: None
# RETURN: Hash of nodes that the user wants to view
# DESC:   Looks for the users enviornment variable for '$NODES' and then checks
#         to see if it is a node list (\W delim) or a file of defined nodes
#         that the user wants to view.

   my ( %nodelist);
   if ( $ENV{NODES} ) {
      if ( -f $ENV{NODES} ) {
         open(NODES, "< $ENV{NODES}");
         while (<NODES>) {
            chomp;
            $_ =~ s/#.*$//;
            $_ =~ s/\W.*$//;
            next unless $_;
            $nodelist{$_} = '1';
         }
         close $NODES;
      } else {
         @s = split(/\W/, $ENV{NODES});
         foreach ( @s ) {
            $nodelist{$_} = '1';
         }
      }
   }
   return(%nodelist);
}


sub node_addr {
   my ($node_num, $network, $mask, @null) = @_;
   my $net_bin = unpack("N", inet_aton("$network"));
   my $mask_bin = unpack("N", inet_aton("$mask"));
   my $twiddle = ((~$mask_bin + 1) & 0xffffffff) >> 2;
   my $addr = ( $net_bin & $mask_bin ) | ( ( $node_num + $twiddle ) & ~$mask_bin );
   $addr = inet_ntoa(pack('N',"$addr"));
   return $addr;
}

sub node_addr_start {
   my ($node_num, $network, $mask, @null) = @_;
   my $net_bin = unpack("N", inet_aton("$network"));
   my $mask_bin = unpack("N", inet_aton("$mask"));
   my $addr = ( $net_bin & $mask_bin ) | ( $node_num & ~$mask_bin );
   $addr = inet_ntoa(pack('N',"$addr"));
   return $addr;
}

1;
